""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class CollapseParams(CoreObject):
    """Parameters to collapse face elements.
    """
    _default_params = {}

    def __initialize(
            self,
            feature_type: SurfaceFeatureType,
            collapse_ratio: float,
            preserve_quality: bool,
            target_skewness: float):
        self._feature_type = SurfaceFeatureType(feature_type)
        self._collapse_ratio = collapse_ratio
        self._preserve_quality = preserve_quality
        self._target_skewness = target_skewness

    def __init__(
            self,
            model: CommunicationManager=None,
            feature_type: SurfaceFeatureType = None,
            collapse_ratio: float = None,
            preserve_quality: bool = None,
            target_skewness: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CollapseParams.

        Parameters
        ----------
        model: Model
            Model to create a CollapseParams object with default parameters.
        feature_type: SurfaceFeatureType, optional
            Feature type to be preserved when performing collapse.
        collapse_ratio: float, optional
            Maximum ratio of shortest face edge length to longest face edge length.
        preserve_quality: bool, optional
            Option to preserve quality of neighboring triangles when performing collapse.Collapse may lead to quality deterioration beyond target skewness. Such collapse is prevented, when the option is enabled.
        target_skewness: float, optional
            Skewness limit used as target to preserve quality. Better quality elements are skipped for collapse.
        json_data: dict, optional
            JSON dictionary to create a CollapseParams object with provided parameters.

        Examples
        --------
        >>> collapse_params = prime.CollapseParams(model = model)
        """
        if json_data:
            self.__initialize(
                SurfaceFeatureType(json_data["featureType"] if "featureType" in json_data else None),
                json_data["collapseRatio"] if "collapseRatio" in json_data else None,
                json_data["preserveQuality"] if "preserveQuality" in json_data else None,
                json_data["targetSkewness"] if "targetSkewness" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [feature_type, collapse_ratio, preserve_quality, target_skewness])
            if all_field_specified:
                self.__initialize(
                    feature_type,
                    collapse_ratio,
                    preserve_quality,
                    target_skewness)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CollapseParams")
                    json_data = param_json["CollapseParams"] if "CollapseParams" in param_json else {}
                    self.__initialize(
                        feature_type if feature_type is not None else ( CollapseParams._default_params["feature_type"] if "feature_type" in CollapseParams._default_params else SurfaceFeatureType(json_data["featureType"] if "featureType" in json_data else None)),
                        collapse_ratio if collapse_ratio is not None else ( CollapseParams._default_params["collapse_ratio"] if "collapse_ratio" in CollapseParams._default_params else (json_data["collapseRatio"] if "collapseRatio" in json_data else None)),
                        preserve_quality if preserve_quality is not None else ( CollapseParams._default_params["preserve_quality"] if "preserve_quality" in CollapseParams._default_params else (json_data["preserveQuality"] if "preserveQuality" in json_data else None)),
                        target_skewness if target_skewness is not None else ( CollapseParams._default_params["target_skewness"] if "target_skewness" in CollapseParams._default_params else (json_data["targetSkewness"] if "targetSkewness" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            feature_type: SurfaceFeatureType = None,
            collapse_ratio: float = None,
            preserve_quality: bool = None,
            target_skewness: float = None):
        """Set the default values of CollapseParams.

        Parameters
        ----------
        feature_type: SurfaceFeatureType, optional
            Feature type to be preserved when performing collapse.
        collapse_ratio: float, optional
            Maximum ratio of shortest face edge length to longest face edge length.
        preserve_quality: bool, optional
            Option to preserve quality of neighboring triangles when performing collapse.Collapse may lead to quality deterioration beyond target skewness. Such collapse is prevented, when the option is enabled.
        target_skewness: float, optional
            Skewness limit used as target to preserve quality. Better quality elements are skipped for collapse.
        """
        args = locals()
        [CollapseParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CollapseParams.

        Examples
        --------
        >>> CollapseParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CollapseParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._feature_type is not None:
            json_data["featureType"] = self._feature_type
        if self._collapse_ratio is not None:
            json_data["collapseRatio"] = self._collapse_ratio
        if self._preserve_quality is not None:
            json_data["preserveQuality"] = self._preserve_quality
        if self._target_skewness is not None:
            json_data["targetSkewness"] = self._target_skewness
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "feature_type :  %s\ncollapse_ratio :  %s\npreserve_quality :  %s\ntarget_skewness :  %s" % (self._feature_type, self._collapse_ratio, self._preserve_quality, self._target_skewness)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def feature_type(self) -> SurfaceFeatureType:
        """Feature type to be preserved when performing collapse.
        """
        return self._feature_type

    @feature_type.setter
    def feature_type(self, value: SurfaceFeatureType):
        self._feature_type = value

    @property
    def collapse_ratio(self) -> float:
        """Maximum ratio of shortest face edge length to longest face edge length.
        """
        return self._collapse_ratio

    @collapse_ratio.setter
    def collapse_ratio(self, value: float):
        self._collapse_ratio = value

    @property
    def preserve_quality(self) -> bool:
        """Option to preserve quality of neighboring triangles when performing collapse.Collapse may lead to quality deterioration beyond target skewness. Such collapse is prevented, when the option is enabled.
        """
        return self._preserve_quality

    @preserve_quality.setter
    def preserve_quality(self, value: bool):
        self._preserve_quality = value

    @property
    def target_skewness(self) -> float:
        """Skewness limit used as target to preserve quality. Better quality elements are skipped for collapse.
        """
        return self._target_skewness

    @target_skewness.setter
    def target_skewness(self, value: float):
        self._target_skewness = value

class CollapseResults(CoreObject):
    """Results associated with collapse face elements.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            n_collapsed: int,
            n_splits: int):
        self._error_code = ErrorCode(error_code)
        self._n_collapsed = n_collapsed
        self._n_splits = n_splits

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            n_collapsed: int = None,
            n_splits: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CollapseResults.

        Parameters
        ----------
        model: Model
            Model to create a CollapseResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        n_collapsed: int, optional
            Number of face elements collapsed.
        n_splits: int, optional
            Number of face elements split.
        json_data: dict, optional
            JSON dictionary to create a CollapseResults object with provided parameters.

        Examples
        --------
        >>> collapse_results = prime.CollapseResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["nCollapsed"] if "nCollapsed" in json_data else None,
                json_data["nSplits"] if "nSplits" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, n_collapsed, n_splits])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    n_collapsed,
                    n_splits)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CollapseResults")
                    json_data = param_json["CollapseResults"] if "CollapseResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( CollapseResults._default_params["error_code"] if "error_code" in CollapseResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        n_collapsed if n_collapsed is not None else ( CollapseResults._default_params["n_collapsed"] if "n_collapsed" in CollapseResults._default_params else (json_data["nCollapsed"] if "nCollapsed" in json_data else None)),
                        n_splits if n_splits is not None else ( CollapseResults._default_params["n_splits"] if "n_splits" in CollapseResults._default_params else (json_data["nSplits"] if "nSplits" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            n_collapsed: int = None,
            n_splits: int = None):
        """Set the default values of CollapseResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the failure of operation.
        n_collapsed: int, optional
            Number of face elements collapsed.
        n_splits: int, optional
            Number of face elements split.
        """
        args = locals()
        [CollapseResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CollapseResults.

        Examples
        --------
        >>> CollapseResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CollapseResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._n_collapsed is not None:
            json_data["nCollapsed"] = self._n_collapsed
        if self._n_splits is not None:
            json_data["nSplits"] = self._n_splits
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nn_collapsed :  %s\nn_splits :  %s" % (self._error_code, self._n_collapsed, self._n_splits)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def n_collapsed(self) -> int:
        """Number of face elements collapsed.
        """
        return self._n_collapsed

    @n_collapsed.setter
    def n_collapsed(self, value: int):
        self._n_collapsed = value

    @property
    def n_splits(self) -> int:
        """Number of face elements split.
        """
        return self._n_splits

    @n_splits.setter
    def n_splits(self, value: int):
        self._n_splits = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class DeleteTool(CoreObject):
    """Performs various delete operation. For example, delete fringes and overlapping faces.

    """

    def __init__(self, model: CommunicationManager):
        """ Initialize DeleteTool """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::DeleteTool/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for DeleteTool. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for DeleteTool. """
        command_name = "PrimeMesh::DeleteTool/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def delete_fringes_and_overlaps_on_zonelets(self, part_id : int, face_zonelets : Iterable[int], params : DeleteFringesAndOverlapsParams) -> DeleteFringesAndOverlapsResults:
        """ Delete fringes and overlapping faces on the given face zonelets.


        Parameters
        ----------
        part_id : int
            Id of a part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        params : DeleteFringesAndOverlapsParams
            Parameters to delete the fringes and overlapping faces.

        Returns
        -------
        DeleteFringesAndOverlapsResults
            Returns the DeleteFringesAndOverlapsResults.

        Examples
        --------
        >>> delete_tool = prime.DeleteTool(model = model)
        >>> results = delete_tool.delete_fringes_and_overlaps_on_zonelets(
        >>>                                    part.id,
        >>>                                    part.get_face_zonelets(),
        >>>                                    prime.DeleteFringesAndOverlapsParams(
        >>>                                       model=model))

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for part_id, valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for face_zonelets, valid argument type is Iterable[int].")
        if not isinstance(params, DeleteFringesAndOverlapsParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is DeleteFringesAndOverlapsParams.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::DeleteTool/DeleteFringesAndOverlapsOnZonelets"
        self._model._print_logs_before_command("delete_fringes_and_overlaps_on_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_fringes_and_overlaps_on_zonelets", DeleteFringesAndOverlapsResults(model = self._model, json_data = result))
        return DeleteFringesAndOverlapsResults(model = self._model, json_data = result)



"""Module containing classes and methods related to multi-zone control."""
from ansys.meshing.prime.autogen.multizonecontrol import (
    MultiZoneControl as _MultiZoneControl,
)
from ansys.meshing.prime.internals.comm_manager import CommunicationManager


class MultiZoneControl(_MultiZoneControl):
    """MultiZone Control to describe parameters and controls used for MultiZone meshing."""

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """Initialize class variables and the superclass."""
        self._model = model
        _MultiZoneControl.__init__(self, model, id, object_id, name)

    def __str__(self) -> str:
        """Provide a string implementation for control."""
        return "Not implemented yet"



"""Implementation of Scanner for scanning bytes or bytearrays."""
import re
from typing import Callable, Tuple

NUMBER_RE = re.compile(
    b'(-?(?:0|[1-9]\d*))(\.\d+)?([eE][-+]?\d+)?', (re.VERBOSE | re.MULTILINE | re.DOTALL)
)


def make_scanner(context) -> Callable[[str, int], Tuple[str, int]]:
    """Make a scanner for bytes or bytearrays.

    Parameters
    ----------
    context : JSONDecoder
        Context needed to know the types.

    Returns
    -------
    Callable[[str, int], Tuple[str, int]]
        Function that scans byte or a bytearray.

    Raises
    ------
    StopIteration
        Stop the current iteration.
    StopIteration
        Stop the current iteration.
    """
    parse_object = context.parse_object
    parse_array = context.parse_array
    parse_string = context.parse_string
    match_number = NUMBER_RE.match
    parse_float = context.parse_float
    parse_int = context.parse_int
    parse_constant = context.parse_constant
    object_hook = context.object_hook
    object_pairs_hook = context.object_pairs_hook
    parse_bytes = context.parse_bytes
    memo = context.memo

    def _scan_once(string: bytes or bytearray, idx):
        try:
            nextchar = bytes(string[idx : idx + 1])
        except IndexError:
            raise StopIteration(idx) from None

        if nextchar == b'"':
            return parse_string(string, idx + 1)
        elif nextchar == b'{':
            return parse_object((string, idx + 1), _scan_once, object_hook, object_pairs_hook, memo)
        elif nextchar == b'[':
            return parse_array((string, idx + 1), _scan_once)
        elif nextchar == b'<':
            return parse_bytes(string, idx + 1)
        elif nextchar == b'n' and string[idx : idx + 4] == b'null':
            return None, idx + 4
        elif nextchar == b't' and string[idx : idx + 4] == b'true':
            return True, idx + 4
        elif nextchar == b'f' and string[idx : idx + 5] == b'false':
            return False, idx + 5

        m = match_number(string, idx)
        if m is not None:
            integer, frac, exp = m.groups()
            if frac or exp:
                res = parse_float(
                    integer.decode('utf-8')
                    + (frac or b'').decode('utf-8')
                    + (exp or b'').decode('utf-8')
                )
            else:
                res = parse_int(integer)
            return res, m.end()
        elif nextchar == b'N' and string[idx : idx + 3] == b'NaN':
            return parse_constant('NaN'), idx + 3
        elif nextchar == b'I' and string[idx : idx + 8] == b'Infinity':
            return parse_constant('Infinity'), idx + 8
        elif nextchar == b'-' and string[idx : idx + 9] == b'-Infinity':
            return parse_constant('-Infinity'), idx + 9
        else:
            raise StopIteration(idx)

    def scan_once(string: bytes or bytearray, idx):
        try:
            return _scan_once(string, idx)
        finally:
            memo.clear()

    return scan_once



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class TopoData(CoreObject):
    """Topodata has all information about connectivity of nodes, edges, elements and more.

    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize TopoData """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def get_geom_zonelets_of_topo_edges(self, topo_edges : Iterable[int]) -> Iterable[int]:
        """ Get the geometry edge zonelets for the provided topoedge ids.


        Parameters
        ----------
        topo_edges : Iterable[int]
            Ids of the topoedges.

        Returns
        -------
        Iterable[int]
            Return the geometry edge zonelet ids.


        Examples
        --------
        >>> geom_edge_zonelets = topo_data.get_geom_zonelets_of_topo_edges(topo_edges)

        """
        if not isinstance(topo_edges, Iterable):
            raise TypeError("Invalid argument type passed for topo_edges, valid argument type is Iterable[int].")
        args = {"topo_edges" : topo_edges}
        command_name = "PrimeMesh::TopoData/GetGeomZoneletsOfTopoEdges"
        self._model._print_logs_before_command("get_geom_zonelets_of_topo_edges", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_geom_zonelets_of_topo_edges")
        return result

    def get_geom_zonelets_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the geometry face zonelets for the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topofaces.

        Returns
        -------
        Iterable[int]
            Return the geometry face zonelet ids.


        Examples
        --------
        >>> geom_face_zonelets = topo_data.get_geom_zonelets_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetGeomZoneletsOfTopoFaces"
        self._model._print_logs_before_command("get_geom_zonelets_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_geom_zonelets_of_topo_faces")
        return result

    def get_mesh_zonelets_of_topo_edges(self, topo_edges : Iterable[int]) -> Iterable[int]:
        """ Get the mesh edge zonelets for the provided topoedge ids.


        Parameters
        ----------
        topo_edges : Iterable[int]
            Ids of the topoedges.

        Returns
        -------
        Iterable[int]
            Return the mesh edge zonelet ids.


        Examples
        --------
        >>> mesh_edge_zonelets = topo_data.get_mesh_zonelets_of_topo_edges(topo_edges)

        """
        if not isinstance(topo_edges, Iterable):
            raise TypeError("Invalid argument type passed for topo_edges, valid argument type is Iterable[int].")
        args = {"topo_edges" : topo_edges}
        command_name = "PrimeMesh::TopoData/GetMeshZoneletsOfTopoEdges"
        self._model._print_logs_before_command("get_mesh_zonelets_of_topo_edges", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_mesh_zonelets_of_topo_edges")
        return result

    def get_mesh_zonelets_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the mesh face zonelets for the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topofaces.

        Returns
        -------
        Iterable[int]
            Return the mesh face zonelet ids.


        Examples
        --------
        >>> mesh_face_zonelets = topo_data.get_mesh_zonelets_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetMeshZoneletsOfTopoFaces"
        self._model._print_logs_before_command("get_mesh_zonelets_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_mesh_zonelets_of_topo_faces")
        return result

    def get_topo_edges_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the topoedges of the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topoface.

        Returns
        -------
        Iterable[int]
            Returns the list of topoedge ids.


        Examples
        --------
        >>> topo_edges_of_topo_faces = topo_data.get_topo_edges_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetTopoEdgesOfTopoFaces"
        self._model._print_logs_before_command("get_topo_edges_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_topo_edges_of_topo_faces")
        return result

    def get_adjacent_topo_faces_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the adjacent topofaces for the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topoface.

        Returns
        -------
        Iterable[int]
            Returns the list of adjacent topoface ids.


        Examples
        --------
        >>> topo_faces_of_topo_faces = topo_data.get_adjacent_topo_faces_of_topo_faces(topo_faces)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetAdjacentTopoFacesOfTopoFaces"
        self._model._print_logs_before_command("get_adjacent_topo_faces_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_adjacent_topo_faces_of_topo_faces")
        return result

    def get_adjacent_topo_edges_of_topo_edges(self, topo_edges : Iterable[int]) -> Iterable[int]:
        """ Get the adjacent topoedges for the provided topoedge ids.


        Parameters
        ----------
        topo_edges : Iterable[int]
            Ids of the topoedge.

        Returns
        -------
        Iterable[int]
            Returns the list of adjacent topoedge ids.


        Examples
        --------
        >>> topo_edges_of_topo_edges = topo_data.get_adjacent_topo_edges_of_topo_edges(topo_edges)

        """
        if not isinstance(topo_edges, Iterable):
            raise TypeError("Invalid argument type passed for topo_edges, valid argument type is Iterable[int].")
        args = {"topo_edges" : topo_edges}
        command_name = "PrimeMesh::TopoData/GetAdjacentTopoEdgesOfTopoEdges"
        self._model._print_logs_before_command("get_adjacent_topo_edges_of_topo_edges", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_adjacent_topo_edges_of_topo_edges")
        return result

    def delete_mesh_on_topo_faces(self, topo_faces : Iterable[int], params : DeleteMeshParams) -> DeleteMeshResults:
        """ Delete mesh on the provided topofaces.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of topofaces.
        params : DeleteMeshParams
            Parameters to delete mesh on topofaces.

        Returns
        -------
        DeleteMeshResults
            Returns the DeleteMeshResults.


        Examples
        --------
        >>> params = prime.DeleteMeshParams(model = model)
        >>> result = topo_data.delete_mesh_on_topo_faces(top_faces, params)

        """
        if not isinstance(topo_faces, Iterable):
            raise TypeError("Invalid argument type passed for topo_faces, valid argument type is Iterable[int].")
        if not isinstance(params, DeleteMeshParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is DeleteMeshParams.")
        args = {"topo_faces" : topo_faces,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::TopoData/DeleteMeshOnTopoFaces"
        self._model._print_logs_before_command("delete_mesh_on_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_mesh_on_topo_faces", DeleteMeshResults(model = self._model, json_data = result))
        return DeleteMeshResults(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of TopoData."""
        return self._id

    @property
    def name(self):
        """ Get the name of TopoData."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class FeatureExtraction(CoreObject):
    """Provide functions for all feature extraction operations like extracting edges zonlelets, tracing node paths.

    """

    def __init__(self, model: CommunicationManager):
        """ Initialize FeatureExtraction """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::FeatureExtraction/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for FeatureExtraction. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for FeatureExtraction. """
        command_name = "PrimeMesh::FeatureExtraction/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def extract_features_on_face_zonelets(self, part_id : int, face_zonelets : Iterable[int], params : ExtractFeatureParams) -> ExtractFeatureResults:
        """ Extract edges by angle and face zonelet boundary using given extract feature parameters.


        Parameters
        ----------
        part_id : int
            Id of input part.
        face_zonelets : Iterable[int]
            Ids of input face zonelets.
        params : ExtractFeatureParams
            Parameters used to extract edges.

        Returns
        -------
        ExtractFeatureResults
            Returns a ExtractFeatureResults.


        Examples
        --------
        >>> results = feature_extraction.extract_features_on_face_zonelets(part_id, face_zonelets, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for part_id, valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for face_zonelets, valid argument type is Iterable[int].")
        if not isinstance(params, ExtractFeatureParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is ExtractFeatureParams.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FeatureExtraction/ExtractFeaturesOnFaceZonelets"
        self._model._print_logs_before_command("extract_features_on_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("extract_features_on_face_zonelets", ExtractFeatureResults(model = self._model, json_data = result))
        return ExtractFeatureResults(model = self._model, json_data = result)

    def create_intersection_edge_loops(self, part_face_zonelets : List[PartZonelets], intersecting_part_face_zonelets : List[PartZonelets], params : CreateIntersectionEdgeLoopsParams) -> CreateIntersectionEdgeLoopsResults:
        """ Finds the edge zonelets formed by intersection of two face zonelets. Performs n to n intersection of face zonelets present in part face zonelets with those in intersecting part face zonelets depending on the input parameters. Order of input does not matter.


        Parameters
        ----------
        part_face_zonelets : PartZoneletsArray
            List of part zonelets to be intersected.
        intersecting_part_face_zonelets : PartZoneletsArray
            List of part zonelets to intersect with.
        params : CreateIntersectionEdgeLoopsParams
            Parameter to control edge extraction.

        Returns
        -------
        CreateIntersectionEdgeLoopsResults
            Returns the CreateIntersectionEdgeLoopsResults.


        Examples
        --------
        >>> results = feature_extraction.create_intersection_edge_loops(part_face_zonelets, intersecting_part_face_zonelets, params)

        """
        if not isinstance(part_face_zonelets, List):
            raise TypeError("Invalid argument type passed for part_face_zonelets, valid argument type is List[PartZonelets].")
        if not isinstance(intersecting_part_face_zonelets, List):
            raise TypeError("Invalid argument type passed for intersecting_part_face_zonelets, valid argument type is List[PartZonelets].")
        if not isinstance(params, CreateIntersectionEdgeLoopsParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is CreateIntersectionEdgeLoopsParams.")
        args = {"part_face_zonelets" : [p._jsonify() for p in part_face_zonelets],
        "intersecting_part_face_zonelets" : [p._jsonify() for p in intersecting_part_face_zonelets],
        "params" : params._jsonify()}
        command_name = "PrimeMesh::FeatureExtraction/CreateIntersectionEdgeLoops"
        self._model._print_logs_before_command("create_intersection_edge_loops", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_intersection_edge_loops", CreateIntersectionEdgeLoopsResults(model = self._model, json_data = result))
        return CreateIntersectionEdgeLoopsResults(model = self._model, json_data = result)



from .download_utilities import DownloadManager
from .examples import (
    download_bracket_dsco,
    download_bracket_fmd,
    download_bracket_scdoc,
    download_deformed_blade_dsco,
    download_deformed_blade_fmd,
    download_deformed_blade_scdoc,
    download_elbow_dsco,
    download_elbow_fmd,
    download_elbow_pmdat,
    download_elbow_scdoc,
    download_f1_rw_drs_stl,
    download_f1_rw_enclosure_stl,
    download_f1_rw_end_plates_stl,
    download_f1_rw_main_plane_stl,
    download_pcb_pmdat,
    download_pcb_scdoc,
    download_pipe_tee_dsco,
    download_pipe_tee_fmd,
    download_pipe_tee_pmdat,
    download_pipe_tee_scdoc,
    download_saddle_bracket_dsco,
    download_saddle_bracket_fmd,
    download_saddle_bracket_scdoc,
    download_toy_car_dsco,
    download_toy_car_fmd,
    download_toy_car_pmdat,
    download_toy_car_scdoc,
    download_turbine_blade_cdb,
)
from .unit_test_examples import download_test_examples



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class VolumeSearch(CoreObject):
    """VolumeSearch allows you to check volume mesh quality.

    VolumeSearch performs volume mesh quality check based on different cell quality measures.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize VolumeSearch """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::VolumeSearch/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for VolumeSearch. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for VolumeSearch. """
        command_name = "PrimeMesh::VolumeSearch/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def get_volume_quality_summary(self, params : VolumeQualitySummaryParams) -> VolumeQualitySummaryResults:
        """ Gets the volume quality summary.

        Diagnose volume quality for the given scope and cell quality measures provided in the VolumeQualitySummaryParams structure.
        Use default quality limit if the parameters are not specified.

        Parameters
        ----------
        params : VolumeQualitySummaryParams
            Volume quality summary parameters.

        Returns
        -------
        VolumeQualitySummaryResults
            Returns the VolumeQualitySummaryResults.

        Examples
        --------
        >>> vol_search = VolumeSearch(model=model)
        >>> results = vol_search.get_volume_quality_summary(VolumeQualitySummaryParams(model=model))

        """
        if not isinstance(params, VolumeQualitySummaryParams):
            raise TypeError("Invalid argument type passed for params, valid argument type is VolumeQualitySummaryParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::VolumeSearch/GetVolumeQualitySummary"
        self._model._print_logs_before_command("get_volume_quality_summary", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volume_quality_summary", VolumeQualitySummaryResults(model = self._model, json_data = result))
        return VolumeQualitySummaryResults(model = self._model, json_data = result)
